(:~
 : This is the main XQuery which will (by default) be called by controller.xql
 : to process any URI ending with ".html". It receives the HTML from
 : the controller and passes it to the templating system.
 :)
xquery version "3.1";

import module namespace templates="http://exist-db.org/xquery/templates" ;
import module namespace mviewer="http://sade/multiviewer" at "multiviewer/multiviewer.xqm" ;
import module namespace f-misc="http://fontane-nb.dariah.eu/ns/SADE/misc" at "fontane/misc.xqm";
import module namespace f-indexviewer="http://fontane-nb.dariah.eu/indexviewer" at "fontane/index-viewer.xqm";
import module namespace f-render="http://fontane-nb.dariah.eu/ns/SADE/render" at "fontane/render.xqm";
import module namespace config="http://textgrid.de/ns/SADE/config" at "config/config.xqm";
import module namespace fsearch = "http://sade/faceted-search" at "faceted-search/faceted-search.xqm";
import module namespace nav="http://sade/tmpl-nav" at "tmpl-nav/tmpl-nav.xqm";
import module namespace wiki="http://textgrid.de/ns/SADE/wiki" at "wiki/wiki.xqm";

(:
 : The following modules provide functions which will be called by the
 : templating.
 :)
import module namespace app="http://textgrid.de/ns/SADE/templates" at "app.xql";

declare namespace output = "http://www.w3.org/2010/xslt-xquery-serialization";

declare option output:method "html5";
declare option output:media-type "text/html";

let $config := map {
    $templates:CONFIG_APP_ROOT : $config:app-root,
    $templates:CONFIG_STOP_ON_ERROR : true()
}
(:
 : We have to provide a lookup function to templates:apply to help it
 : find functions in the imported application modules. The templates
 : module cannot see the application modules, but the inline function
 : below does see them.
 :)
let $lookup := function($functionName as xs:string, $arity as xs:int) {
    try {
        function-lookup(xs:QName($functionName), $arity)
    } catch * {
        ()
    }
}
(:
 : The HTML is passed in the request from the controller.
 : Run it through the templating system and return the result.
 :)

let $project-path := request:get-url() => substring-after("/SADE/") => tokenize("/")
let $project := $project-path[1]
let $resource := $project-path[last()]
let $doc-path := $config:projects-dir || $project || config:get("template") || $resource

let $content :=
    if( doc-available($doc-path)) then doc( $doc-path )
    else
        (doc($config:projects-dir || $project || config:get("template") || "/error.html"),
        response:set-status-code( 404 )
        )

return
    templates:apply($content, $lookup, (), $config)
